GENOME_FILES = {
    "ensembl_hg38.chrom.sizes": "hg38",
    "ncbi_hg38.chrom.sizes": "hg38",
    "ucsc_dm6.chrom.sizes": "dm6",
    "ucsc_hg19.chrom.sizes": "hg19",
    "ucsc_hg38.chrom.sizes": "hg38",
    "ucsc_mm10.chrom.sizes": "mm10",
    "ucsc_mm39.chrom.sizes": "mm39",
    "ucsc_mm9.chrom.sizes": "mm9",
    "ucsc_panTo6.chrom.sizes": "panTro6",
}
